﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using System.Text;
using System.Xml;

using dp2Circulation;
//如果是在dp2Catalog前端使用，请把上行注释掉，启用下行引用
//using dp2Catalog;

using DigitalPlatform.Marc;
using DigitalPlatform.Xml;
using DigitalPlatform.Script;
using DigitalPlatform.IO;
using DigitalPlatform.dp2.Statis;


public class MyMarcQueryHost : MarcQueryHost
{
    ExcelDocument doc = null;
    int _lineIndex = 0;

    public override void OnBegin(object sender, StatisEventArgs e)
    {
        // 询问文件名 
        SaveFileDialog dlg = new SaveFileDialog();
        dlg.Title = "请指定要输出的 Excel 文件名";
        dlg.CreatePrompt = false;
        dlg.OverwritePrompt = true;
        dlg.Filter = "Excel 文件 (*.xlsx)|*.xlsx|All files (*.*)|*.*";
        dlg.RestoreDirectory = true;
        if (dlg.ShowDialog() != DialogResult.OK)
        {
            e.Continue = ContinueType.SkipAll;
            return;
        }

        doc = ExcelDocument.Create(dlg.FileName);
        doc.NewSheet("Sheet1");
        WriteTitleLine();
    }

    void WriteTitleLine()
    {
        string[] titles = new string[] { 
            "ID",
            "ISBN",
            "书名",
            "定价",
            "CIP分类",
            "内容简介",
            "作者",
            "出版社",
            "版次",
            "出版时间",
            "主题词",
        };

        int nColIndex = 0;

        foreach (string t in titles)
        {
            doc.WriteExcelCell(this._lineIndex, nColIndex++, t, true);
        }

        this._lineIndex++;
    }

    public override void FreeResources()
    {
        if (doc != null)
        {
            doc.Close();
            doc = null;
        }
        base.FreeResources();
    }

    string GetContent(MarcRecord record, string strFieldName, string strSubfieldName)
    {
        string strValue = "";
        MarcNodeList fields = record.select("field[@name=" + strFieldName + "]");
        foreach (MarcNode field in fields)
        {
            MarcNodeList subfields = field.select("subfield[@name='" + strSubfieldName + "']");
            foreach (MarcNode subfield in subfields)
            {
                if (strValue != "")
                    strValue += ";";

                strValue += subfield.Content;
            }
        }

        return strValue;
    }


	public override void OnRecord(object sender, StatisEventArgs e)
	{
        int nColIndex = 0; 
        List<CellData> cells = new List<CellData>(); // 单元数组

	MarcRecord record = this.MarcRecord;
        // ID
        string strId = this.RecordPath;
        string[] parts = strId.Split(new char[] { '/','@' });
        if (parts.Length == 3)
            strId = parts[1].PadLeft(10, '0');
        cells.Add(new CellData(nColIndex++, strId));

        // ISBN 
        string strISBN = GetContent(record, "010", "a");
        cells.Add(new CellData(nColIndex++, strISBN, true, 0));

        // 书名 
        string strTitle = GetContent(record, "200", "a");
        cells.Add(new CellData(nColIndex++, strTitle));

        // 价格 
        string strPrice = GetContent(record, "010", "d");
        cells.Add(new CellData(nColIndex++, strPrice));


        // CIP分类
        string strCIP = GetContent(record, "690", "a");
        cells.Add(new CellData(nColIndex++, strCIP));

        // 内容简介
        string strSummary = GetContent(record, "330", "a");
        cells.Add(new CellData(nColIndex++, strSummary));

        // 作者 
        string strAuthor = GetContent(record, "200", "f");
        cells.Add(new CellData(nColIndex++, strAuthor));

        // 出版者 
        string strPublisher = GetContent(record, "210", "c");
        cells.Add(new CellData(nColIndex++, strPublisher));

        // 版次
        string strRevision = GetContent(record, "205", "a");
        cells.Add(new CellData(nColIndex++, strRevision));

        // 出版日期 
        string strPublishDate = GetContent(record, "210", "d");
        cells.Add(new CellData(nColIndex++, strPublishDate));

        // 主题词
        string strSubject = GetContent(record, "606", "a");
        cells.Add(new CellData(nColIndex++, strSubject));


        doc.WriteExcelLine(this._lineIndex, cells, WriteExcelLineStyle.None); // 以字符串方式写入
        doc.SaveWorksheet();
        this._lineIndex++;
	}
}
